<?php

if (!function_exists('add_action')) {
  die('Please don\'t open this file directly!');
}

require_once 'sn-do-actions.php';


class wf_sn_do {
  static function format_size($bytes) {
    if ($bytes > 1073741824) {
      return number_format_i18n($bytes / 1073741824, 2) . ' GB';
    } elseif ($bytes > 1048576) {
      return number_format_i18n($bytes / 1048576, 1) . ' MB';
    } elseif ($bytes > 1024) {
      return number_format_i18n($bytes / 1024, 1) . ' KB';
    } else {
      return number_format_i18n($bytes, 0) . ' bytes';
    }
  } // format_size
  

  // display results
  static function do_page() {
    global $wpdb;
    
    echo '<div class="submit-test-container">';

    $tables = $size = $records = $overhead = 0;
    
    $table_status = $wpdb->get_results('SHOW TABLE STATUS');
    $table_prefix = $wpdb->get_blog_prefix();
    
    if (is_array($table_status)) {
      foreach ($table_status as $index => $table) {
        if (0 !== stripos($table->Name, $table_prefix)) {
          continue;
        }
        if ($table->Engine == 'MyISAM') {
          $overhead += $table->Data_free;
        }
        $tables++;
        $records += $table->Rows;
        $size += $table->Data_length + $table->Index_length;
      }
    }
    
    echo '<div id="counters">';
    echo '<span class="score" style="border-left: none;">' . number_format_i18n($tables) . '<br><i>Tables</i></span>';
    echo '<span class="score">' . self::format_size($size) . '<br><i>Total Data Size</i></span>';  
    echo '<span class="score">' . self::format_size($overhead) . '<br><i>Wasted Space Optimizer Will Recover</i></span>';  
    echo '<span class="score">' . number_format_i18n($records) . '<br><i>Total Records</i></span>';  
    echo '</div>';
    
    echo '<p>As you use WordPress and add more content to your site, it will inevitably lead to <b>garbage data accumulation</b> in your database. Extra database records wont only slow your site down but they will also take up unnecessary space which will lead to <b>higher hosting costs</b>. Security Ninja\'s <b>Database Optimizer</b> module will optimize your database and remove data that slows down your site and that you don\'t need. Run all <b>11 optimizations</b> with a single click and make your <b>site faster</b>.</p>';
    
    if ($size < 1024) {
      echo '<p><a target="_blank" href="' . wf_sn::generate_sn_web_link('tab_database_optimizer') . '" class="button-primary input-button">Optimize ' . number_format_i18n($tables) . ' tables &amp; make your site faster</a></p></div>';  
    } else {
      echo '<p><a target="_blank" href="' . wf_sn::generate_sn_web_link('tab_database_optimizer') . '" class="button-primary input-button">Optimize ' . number_format_i18n($tables) . ' tables, regain ' . self::format_size($overhead) .' of space &amp; make your site faster</a></p></div>';
    }
    
    echo '<table class="wp-list-table widefat" cellspacing="0" id="security-ninja">';
    echo '<thead><tr>';
    echo '<th>Optimization Name</th>';
    echo '<th>Optimization Status</th>';
    echo '<th><a href="javascript: return false;" title="Get Security Ninja PRO to instantly run all optimizations" class="button">Get PRO to Run Optimizations</a></th>';
    echo '</tr></thead>';
    echo '<tbody>';
    
    foreach (wf_sn_do_actions::get_actions() as $action => $name) {
      if (is_callable(array('wf_sn_do_actions', 'info_' . $action))) {
        $desc = call_user_func(array('wf_sn_do_actions', 'info_' . $action));
      } else {
        $desc = 'No info available.';  
      }
      
      echo '<tr>';
      echo '<td>' . $name . '</td>';
      echo '<td class="do-optimization-desc" id="' . $action . '_desc">' . $desc . '</td>';
      echo '<td class="sn-details"><a title="Get Security Ninja PRO to instantly run all optimizations" disabled="disabled" href="javascript: return false;" class="button disabled">Run Optimization</a></td>';
      echo '</tr>';
    } // foreach
    
    
    echo '</tbody>';
    echo '</table><br>';
    echo '<p><a href="#" class="hide_tab" data-tab-id="optimizer" title="Hide this tab"><i>No thank you, I\'m not interested (hide this tab)</i></a></p>';
  } // do_page
} // wf_sn_do class
